IMPLEMENTATION MODULE Alpha;

(*

   General procedures for manipulating alpha data

   Revision history:
     0.00


   Copyright 1987 by:
     Dale W. Thompson, 14500 Dallas Pkwy. #2091, Dallas, TX 75240

     This module and/or its procedures may be freely used by anyone,
     but please acknowledge its use in any copyright notice of a
     publicly distributed program.  Thank you.

     Please forward any comments, problems, or suggestions to me
     at the address given, or to my CompuServe ID 75115,734.
*)


(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsAlpha( c: CHAR ): BOOLEAN;

  BEGIN
    IF (IsUpper(c) OR IsLower(c)) THEN RETURN TRUE END;
    RETURN FALSE;
  END IsAlpha;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsUpper( c: CHAR ): BOOLEAN;

  BEGIN
    RETURN ('A' <= c ) AND ( c <= 'Z' )
  END IsUpper;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsLower( c: CHAR ): BOOLEAN;

  BEGIN
    RETURN ('a' <= c ) AND ( c <= 'z' )
  END IsLower;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsDigit( c: CHAR ): BOOLEAN;

  BEGIN
    IF ('0' <= c) AND (c <= '9') THEN RETURN TRUE END;
    RETURN FALSE;
  END IsDigit;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsXDigit( c: CHAR ): BOOLEAN;

  BEGIN
    IF IsDigit(c) OR (('A' <= c) AND (c <= 'F')) OR
       (('a' <= c) AND (c <= 'f')) THEN RETURN TRUE END;
    RETURN FALSE;
  END IsXDigit;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsAlNum( c: CHAR ): BOOLEAN;

  BEGIN
    IF (IsAlpha(c) OR IsDigit(c)) THEN RETURN TRUE END;
    RETURN FALSE;
  END IsAlNum;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsSpace( c: CHAR ): BOOLEAN;

  BEGIN
    IF (c = 40C) OR ((11C <= c) AND (c <= 15C)) THEN RETURN TRUE END;
    RETURN FALSE;
  END IsSpace;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsPunct( c: CHAR ): BOOLEAN;

  BEGIN
    IF IsControl(c) OR IsAlNum(c) OR IsSpace(c) THEN RETURN FALSE END;
    RETURN TRUE;
  END IsPunct;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsPrint( c: CHAR ): BOOLEAN;

  BEGIN
    IF (40C <= c) AND (c <= 176C) THEN RETURN TRUE END;
    RETURN FALSE;
  END IsPrint;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsGraph( c: CHAR ): BOOLEAN;

  BEGIN
    IF (41C <= c) AND (c <= 176C) THEN RETURN TRUE END;
    RETURN FALSE;
  END IsGraph;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsControl( c: CHAR ): BOOLEAN;

  BEGIN
    IF ((0C <= c) AND (c <= 37C)) OR (c = 177C) THEN RETURN TRUE END;
    RETURN FALSE;
  END IsControl;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsAscii( c: CHAR ): BOOLEAN;

  BEGIN
    IF (c < 200C) THEN RETURN TRUE END;
    RETURN FALSE;
  END IsAscii;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE IsWhite( c: CHAR ): BOOLEAN;

  VAR ct: CharTypeSet;

  BEGIN
    FindCharType( c, ct);
    IF (WHITESPACE IN ct) THEN RETURN TRUE END;
    RETURN FALSE;
  END IsWhite;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE ToLower ( c: CHAR (* in *) ): CHAR;

  BEGIN
    IF IsUpper(c) THEN RETURN CHR( ORD(c) + 32 ) END;
    RETURN c
  END ToLower;

(*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)
PROCEDURE FindCharType (    C  : CHAR        (* in     *);
                        VAR CT : CharTypeSet (* in/out *) );

      (* This procedure returns the CharType of the character passed
         to it.  The possible types are enumerated in the TYPE
         declaration for CharType.
      *)

  VAR c: CARDINAL;

  BEGIN (* PROCEDURE FindCharType *) (* Find and return character type *)

    c := ORD(C);
    CT := CharTypeSet{};


    IF (c >= 0) & (c < 32) THEN
      INCL (CT, CONTROL);
      IF c = 0 THEN
        INCL (CT, ASCNULL);
      ELSIF c = 9 THEN
        INCL (CT,  HT);
        INCL (CT,  WHITESPACE);
      ELSIF c = 10 THEN
        INCL (CT,  LF);
      ELSIF c = 11 THEN
        INCL (CT, VT);
      ELSIF c = 12 THEN
        INCL (CT, FF);
      ELSIF c = 13 THEN
        INCL (CT,  CR);
      ELSIF c = 27 THEN
        INCL (CT, ESCAPE);
      END;
    ELSIF (C >= ' ') & (c < 127) THEN
      INCL (CT,  PRINTABLE);
      IF C = ' ' THEN
        INCL (CT,  SPACE);
        INCL (CT,  WHITESPACE);
      ELSIF C = '.' THEN
        INCL (CT,  PERIOD);
      END;
      IF (C >= '0') & (C <= '9') THEN
        INCL (CT, DIGIT);
        INCL (CT, ALPHANUMERIC);
      ELSIF (C >= 'a') & (C <= 'z') THEN
        INCL (CT, LOWERCASE);
        INCL (CT, ALPHA);
        INCL (CT, ALPHANUMERIC);
      ELSIF (C >= 'A') & (C < 'Z') THEN
        INCL (CT, UPPERCASE);
        INCL (CT, ALPHA);
        INCL (CT, ALPHANUMERIC);
      ELSIF ((C > ' ') & (C <= '/')) OR ((C >= ':') & (C <= '@')) OR
      ((C >= '[') & (c <= 96)) OR ((C >= '{') & (c < 127)) THEN
        INCL (CT, PUNCTUATION);
      END;
    ELSIF c = 127 THEN
      INCL (CT,  DEL);
      INCL (CT, CONTROL);
    ELSIF c > 127 THEN
      INCL (CT, ALTERNATE);
    END;


  END FindCharType;


END (* IMPLEMENTATION MODULE *) Alpha.