MODULE CtoModula;

(*
   A simple C-to-Modula converter.

   The logic for this program began from the book "Advanced Modula-2" by
   Herbert Schildt, but has been expanded quite a bit.

   The code looks and is pretty sloppy, but it seems to work for most
   C programs.  It should eliminate a lot of time in the translation
   process.

   Revision history:
     1.00               Initial Release  4/29/87


   Copyright 1987 by:
     Dale W. Thompson, 14500 Dallas Pkwy. #2091, Dallas, TX 75240

     This module and/or its procedures may be freely used by anyone,
     but please acknowledge its use in any copyright notice of a
     publicly distributed program.  Thank you.

     Please forward any comments, problems, or suggestions to me
     at the address given, or to my CompuServe ID 75115,734.
*)


FROM InOut IMPORT EOL, WriteString, WriteLn, Write, OpenOutput, CloseOutput;
FROM Files IMPORT FileExists;
FROM FileNames IMPORT SplitName;
FROM Strings IMPORT Length;
FROM Strings2 IMPORT Equal;
FROM Alpha IMPORT IsWhite, IsAlpha, IsDigit, FindCharType, CharTypeSet,
                  CharTypes;
FROM FastTextIO IMPORT FastFile, OpenFF, CloseFF, ReadLine, EofFF;
FROM CommandLine IMPORT CLStrings, GetCL;

CONST
  indentlevel = 2; (* acually it's one more than this *)

TYPE
  TType = (DELIMITER, NAME, NUMBER, STR, ENDING);
  LType = (InWhile, InFor, InIf, InElse, InCase, InRecord, InNone);

VAR
   argc, i           : CARDINAL;
   argv              : ARRAY [0..2] OF CLStrings;

VAR
  token, prog : ARRAY [0..254] OF CHAR;
  TokType     : TType;
  LineType    : LType;
  t, indent   : CARDINAL;
  lastt       : CARDINAL;
  dn          : BOOLEAN;
  inf         : FastFile;
  fname       : ARRAY [0..132] OF CHAR;
  CommentLevel: CARDINAL;

PROCEDURE IsDelim(ch: CHAR): BOOLEAN;

  VAR what : CharTypeSet;
  BEGIN
    FindCharType(ch, what);
    IF (PUNCTUATION IN what) OR (WHITESPACE IN what) THEN RETURN TRUE END;
    RETURN FALSE
  END IsDelim;

PROCEDURE GetToken(VAR Indent: CARDINAL; VAR TokenType: TType);
  VAR what: CharTypeSet;
      tt  : CARDINAL;
      tmpc: CHAR;
  BEGIN
    token := ""; tt := 0;(* null string *)
    lastt := t;
    WHILE (IsWhite(prog[t]) AND (t < HIGH(prog))) DO INC(t) END;
    IF (prog[t] = EOL) OR (prog[t] = 0C) OR (t >= HIGH(prog)) THEN

      CASE LineType OF
        InFor:
          token := " DO"; |
        InIf:
          token := " THEN"; |
      ELSE
        token[tt] := 0C;
      END;
      LineType := InNone;
      TokenType:= ENDING;
      INC(t);
    ELSIF (CommentLevel > 0) THEN
      TokenType := STR;
      LOOP
        IF (prog[t] = '*') AND (prog[t+1] = '/') THEN
          DEC(CommentLevel);
          token[tt] := '*'; INC(tt); INC(t);
          token[tt] := ')'; INC(tt); INC(t);
          EXIT;
        ELSIF (prog[t] = EOL) OR (prog[t] = 0C) OR (t = HIGH(prog)) THEN
          EXIT;
        ELSIF (prog[t] = '/') AND (prog[t+1] = '*') THEN
          INC(CommentLevel);
          token[tt] := '('; INC(tt); INC(t);
        END;
        token[tt] := prog[t]; INC(tt); INC(t);
      END;
      token[tt] := 0C;
    ELSE
      FindCharType(prog[t], what);
      IF (PUNCTUATION IN what) THEN
        IF (prog[t] = '{') OR (prog[t] = '}') THEN
          TokenType := NAME;
          IF prog[t] = '{' THEN
            CASE LineType OF
              InFor:
                token := " DO"; |
              InWhile:
                token := " DO"; |
              InIf:
                token := " THEN"; |
              InElse:
                token := " (* DO *)"; |
              InRecord:
                token := " = "; |
            ELSE
              token := "BEGIN";
            END;
            LineType := InNone;
            INC(Indent);
          ELSE
            token := "END;";
            LineType := InNone;
            IF (Indent > 0) THEN DEC(Indent) END;
          END;
        ELSIF (prog[t] = '"') OR (prog[t] = 27C) THEN
          (* is string *)
          tmpc := prog[t];
          INC(t);
          token[tt] := 42C; INC(tt);
          WHILE ((prog[t] <> tmpc) AND (prog[t] <> 0C)) DO
            token[tt] := prog[t]; INC(tt);
            INC(t);
          END;
          TokenType := STR;
          token[tt] := 42C; INC(tt);
          token[tt] := 0C;
        ELSE
          TokenType := DELIMITER;
          token[0] := prog[t];
          token[1] := 0C;
        END;
        INC(t);
      ELSIF IsAlpha(prog[t]) THEN
        WHILE (NOT IsDelim(prog[t])) AND (prog[t] <> 0C) DO
          token[tt] := prog[t]; INC(tt);
          INC(t);
          WHILE (prog[t] = '_') DO INC(t) END;
        END;
        TokenType := NAME;
        token[tt] := 0C;
      ELSIF IsDigit(prog[t]) THEN
        WHILE (NOT IsDelim(prog[t])) AND (prog[t] <> 0C) DO
          token[tt] := prog[t]; INC(tt);
          INC(t);
        END;
        TokenType := NUMBER;
        token[tt] := 0C;
      END;
    END;
  END GetToken;

PROCEDURE PutBack;
  BEGIN
    t := lastt;
  END PutBack;

PROCEDURE Translate;
  VAR
    ind: CARDINAL;
    TT : TType;

  BEGIN
    ind := 0;
    CASE token[0] OF
      ';':
        IF (LineType = InRecord) THEN LineType := InNone END; |
      '~':
        token := "NOT " |
      '=':
        (* look ahead to see if double equals *)
        GetToken(ind,TT);
        IF Equal(token, "=") THEN
          token := "= "
        ELSE
          PutBack; (* restore token to stream *)
          token := ":= "; (* assignment *)
        END; |
      '#':
        (* look ahead to see if this may be #define command *)
        GetToken(ind,TT);
        IF Equal(token, "define") THEN
          token := "CONST "
        ELSE
          PutBack; (* restore token to stream *)
          token := "#";
        END; |
      '-':
        (* look ahead to see if this may be pointer director *)
        GetToken(ind,TT);
        IF Equal(token, ">") THEN
          token := "^."
        ELSE
          PutBack; (* restore token to stream *)
          token := "-";
        END; |
      '/':
        (* look ahead to see if start of comment *)
        GetToken(ind,TT);
        IF Equal(token, "*") THEN
          token := "(* ";
          INC(CommentLevel);
        ELSE
          PutBack; (* restore token to stream *)
          token := "/";
        END; |
      '!':
        GetToken(ind,TT);
        IF Equal(token, "=") THEN
          token := "<>"
        ELSE
          PutBack; (* restore token to stream *)
          token := " NOT ";
        END; |
      '%':
        token := " MOD "; |
      '|':
        GetToken(ind,TT);
        IF NOT Equal(token, "|") THEN
          PutBack; (* restore token to stream *)
        END;
        token := " OR "; |
      '&':
        GetToken(ind,TT);
        IF NOT Equal(token, "&") THEN
          PutBack; (* restore token to stream *)
          token := "&";
        ELSE
        token := " AND ";
        END; |
      ELSE
        (* now check for keywords *)
        IF Equal(token, "switch") THEN
          token := "CASE";
          LineType := InCase;
        ELSIF Equal(token, "while") THEN
          token := "WHILE";
          LineType := InWhile;
        ELSIF Equal(token, "struct") THEN
          token := "RECORD";
          LineType := InRecord;
        ELSIF Equal(token, "int") THEN
          token := "CARDINAL"
        ELSIF Equal(token, "float") THEN
          token := "REAL"
        ELSIF Equal(token, "char") THEN
          token := "CHAR"
        ELSIF Equal(token, "printf") THEN
          token := "WriteX"
        ELSIF Equal(token, "extern") THEN
          token := "IMPORT"
        ELSIF Equal(token, "for") THEN
          token := "FOR";
          LineType := InFor;
        ELSIF Equal(token, "return") THEN
          token := "RETURN"
        ELSIF Equal(token, "break") THEN
          token := "|"
        ELSIF Equal(token, "case") THEN
          token := ""
        ELSIF Equal(token, "sizeof") THEN
          token := "TSIZE"
        ELSIF Equal(token, "if") THEN
          token := "IF";
          LineType := InIf;
        ELSIF Equal(token, "else") THEN
          GetToken(ind,TT);
          IF Equal(token, "if") THEN
            token := "ELSIF";
            LineType := InIf;
          ELSE
            PutBack;
            token := "ELSE";
            LineType := InElse;
          END;
        (* now try some common C functions *)
        ELSIF Equal(token, "fopen") THEN
          token := "DOSFiles.Open"
        ELSIF Equal(token, "fclose") THEN
          token := "DOSFiles.Close"
        ELSIF Equal(token, "fseek") THEN
          token := "DOSFiles.Seek"
        ELSIF Equal(token, "gets") THEN
          token := "ReadLine"
        ELSIF Equal(token, "puts") THEN
          token := "WriteString"
        ELSIF Equal(token, "getchar") THEN
          token := "InOut.Read"
        ELSIF Equal(token, "putchar") THEN
          token := "InOut.Write"
        ELSIF Equal(token, "fread") THEN
          token := "DOSFiles.Read"
        ELSIF Equal(token, "fwrite") THEN
          token := "DOSFiles.Write"
        ELSIF Equal(token, "strcat") THEN
          token := "Strings.Concat"
        ELSIF Equal(token, "strcmp") THEN
          token := "Strings.Compare"
        ELSIF Equal(token, "strcpy") THEN
          token := "Strings.Assign"
        ELSIF Equal(token, "strlen") THEN
          token := "Strings.Length"
        ELSIF Equal(token, "strchr") THEN
          token := "Stings.Pos"
        ELSIF Equal(token, "toupper") THEN
          token := "CAP"
        ELSIF Equal(token, "free") THEN
          token := "DISPOSE"
        ELSIF Equal(token, "register") THEN
          token := ""
        ELSIF Equal(token, "static") THEN
          token := ""
    (*    ELSIF Equal(token, "") THEN
          token := ""
        ELSIF Equal(token, "") THEN
          token := ""
        ELSIF Equal(token, "") THEN
          token := ""
    *)
        END;
    END; (* CASE *)
  END Translate;

PROCEDURE convert;
  BEGIN
    GetToken(indent, TokType);
    (* now indent properly *)
    FOR i := 1 TO indent * indentlevel DO Write(" ") END;
    LOOP
      CASE TokType OF
        STR:
          WriteString(token); |
        NAME:
          Translate;
          WriteString(token);
          Write(" "); |
        DELIMITER:
          Translate;
          WriteString(token); |
        NUMBER:
          WriteString(token); |
        ENDING:
          IF (token[0] <> 0C) THEN WriteString(token) END;
          EXIT; |
        ELSE
      END; (* CASE *)
      GetToken(indent, TokType);
    END;
    WriteLn;
  END convert;

PROCEDURE ShowUsage;
  BEGIN
    WriteString("Usage: CtoMod <infile>"); WriteLn; HALT;
  END ShowUsage;


  BEGIN (* main *)
   IF NOT GetCL(argc, argv) THEN
     WriteString("Could not read Command Line."); WriteLn;
   ELSE
     IF (argc = 0) OR (argv[0][0] = "?") THEN
       ShowUsage;
     ELSE
       IF (NOT FileExists(argv[0])) THEN
         WriteString(argv[0]); WriteString(" does not exist."); WriteLn;
       ELSE
         SplitName(argv[0], fname, prog, token);
         WriteString("MODULE "); WriteString(fname); WriteString(";");
         WriteLn; WriteLn;
         OpenFF( inf, argv[0], dn);
         IF dn THEN
           LineType := InNone;
           indent := 0;
           CommentLevel := 0;
           REPEAT
             t := 0;
             ReadLine(inf, prog);
             convert;
           UNTIL EofFF(inf);
           WriteLn; WriteString("END (* MODULE *) "); WriteString(fname);
           Write("."); WriteLn;
           CloseFF(inf);
         ELSE
           WriteString("Could not open file "); WriteString(argv[0]); WriteLn;
         END;
       END;
     END;
   END;

END (* MODULE *) CtoModula.
